# _*_ coding: utf-8 _*_
#
# hspylib v0.11.123
#
# Package: main.hspylib.core.datasource
"""Package initialization."""

__all__ = [
  'cassandra', 
  'crud_entity', 
  'crud_repository', 
  'crud_service', 
  'db_configuration', 
  'db_repository', 
  'firebase', 
  'identity', 
  'mysql', 
  'postgres', 
  'redis', 
  'sqlite'
]
__version__ = '0.11.123'
