/*
  Python dependencies file
  Project: hspylib
  Created: 15th Jun, 2022

  !! Syntax for dependencies !!
  package: <pkg_name>, version: <latest|versionNum>, space: <user|global>, mode: <gt|ge|eq|ne|compat|none>
*/

/* Application dependencies */

package: requests, version: 2.28.0, space: user
package: urllib3, version: 1.26.9, space: user
package: cryptocode, version: 0.1, space: user
package: cryptography, version: 37.0.2, space: user
package: emoji, version: 1.7.0, space: user
package: getkey, version: 0.6.5, space: user
package: PyMySQL, version: 1.0.2, space: user
package: pyparsing, version: 3.0.9, space: user
package: pyperclip, version: 1.8.2, space: user
package: PyQt5, version: 5.15.6, space: user
package: PyYAML, version: 6.0, space: user
