# _*_ coding: utf-8 _*_
#
# hspylib v0.11.123
#
# Package: main.hspylib.core.datasource.redis
"""Package initialization."""

__all__ = [
  'redis_configuration', 
  'redis_repository'
]
__version__ = '0.11.123'
